@@ -10,46 +10,41 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <string.h>
 #include <errno.h>
+#include <string.h>
+#include <time.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
 #ifdef HAVE_SYS_SELECT_H
+/* For some platforms this must be below sys/types.h. */
 #include <sys/select.h>
 #endif
-
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #ifdef HAVE_WINSOCK
 #include <winsock.h>
 #endif
 
 #include "fcintl.h"
 #include "game.h"		/* game.all_connections */
+#include "hash.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
+#include "packets.h"
 #include "support.h"		/* mystr(n)casecmp */
 
 #include "connection.h"
 
-/* get 'struct conn_list' functions: */
-#define SPECLIST_TAG conn
-#define SPECLIST_TYPE struct connection
-#include "speclist_c.h"
-
 /* String used for connection.addr and related cases to indicate
  * blank/unknown/not-applicable address:
  */
@@ -60,6 +55,8 @@ const char blank_addr_str[] = "---.---.---.---";
    neccesary as removing a random connection while we are iterating through
    a connection list might corrupt the list. */
 int delayed_disconnect = 0;
+
+struct connection *current_connection;
   
 /**************************************************************************
   Command access levels for client-side use; at present, they are only
@@ -93,7 +90,7 @@ enum cmdlevel_id cmdlevel_named(const char *token)
   enum cmdlevel_id i;
   size_t len = strlen(token);
 
-  for (i = 0; i < ALLOW_NUM; ++i) {
+  for (i = 0; i < ALLOW_NUM; i++) {
     if (strncmp(levelnames[i], token, len) == 0) {
       return i;
     }
@@ -120,11 +117,37 @@ void close_socket_set_callback(CLOSE_FUN fun)
   close_callback = fun;
 }
 
+/**************************************************************************
+  Return the the close_callback.
+**************************************************************************/
+CLOSE_FUN close_socket_get_callback(void)
+{
+  return close_callback;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static bool buffer_ensure_free_extra_space(struct socket_packet_buffer *buf,
+					   int extra_space)
+{
+  /* room for more? */
+  if (buf->nsize - buf->ndata < extra_space) {
+    buf->nsize = buf->ndata + extra_space;
+
+    /* added this check so we don't gobble up too much mem */
+    if (buf->nsize > MAX_LEN_BUFFER) {
+      return FALSE;
+    }
+    buf->data = (unsigned char *) fc_realloc(buf->data, buf->nsize);
+  }
+  return TRUE;
+}
 
 /**************************************************************************
   Read data from socket, and check if a packet is ready.
   Returns:
-    -1  :  an error occured - you should close the socket
+    -1  :  an error occurred - you should close the socket
     >0  :  number of bytes read
     =0  :  non-blocking sockets only; no data read, would block
 **************************************************************************/
@@ -132,8 +155,14 @@ int read_socket_data(int sock, struct socket_packet_buffer *buffer)
 {
   int didget;
 
-  didget=my_readsocket(sock, (char *)(buffer->data+buffer->ndata),
-		       MAX_LEN_PACKET-buffer->ndata);
+  if (!buffer_ensure_free_extra_space(buffer, MAX_LEN_PACKET)) {
+    freelog(LOG_ERROR, "can't grow buffer");
+    return -1;
+  }
+
+  freelog(LOG_DEBUG, "try reading %d bytes", buffer->nsize - buffer->ndata);
+  didget = my_readsocket(sock, (char *) (buffer->data + buffer->ndata),
+			 buffer->nsize - buffer->ndata);
 
   if (didget > 0) {
     buffer->ndata+=didget;
@@ -184,7 +213,13 @@ static int write_socket_data(struct connection *pc,
     tv.tv_sec = 0; tv.tv_usec = 0;
 
     if (select(pc->sock+1, NULL, &writefs, &exceptfs, &tv) <= 0) {
-      break;
+      if (errno != EINTR) {
+	break;
+      } else {
+	/* EINTR can happen sometimes, especially when compiling with -pg.
+	 * Generally we just want to run select again. */
+	continue;
+      }
     }
 
     if (FD_ISSET(pc->sock, &exceptfs)) {
@@ -201,6 +236,7 @@ static int write_socket_data(struct connection *pc,
 
     if (FD_ISSET(pc->sock, &writefs)) {
       nblock=MIN(buf->ndata-start, MAX_LEN_PACKET);
+      freelog(LOG_DEBUG,"trying to write %d limit=%d",nblock,limit);
       if((nput=my_writesocket(pc->sock, 
 			      (const char *)buf->data+start, nblock)) == -1) {
 #ifdef NONBLOCKING_SOCKETS
@@ -248,7 +284,7 @@ void flush_connection_send_buffer_all(struct connection *pc)
 /**************************************************************************
   flush'em
 **************************************************************************/
-void flush_connection_send_buffer_packets(struct connection *pc)
+static void flush_connection_send_buffer_packets(struct connection *pc)
 {
   if(pc && pc->used && pc->send_buffer->ndata >= MAX_LEN_PACKET) {
     write_socket_data(pc, pc->send_buffer, MAX_LEN_PACKET-1);
@@ -281,23 +317,17 @@ static bool add_connection_data(struct connection *pc,
 
     buf = pc->send_buffer;
 
-    /* room for more? */
-    if(buf->nsize - buf->ndata < len) {
-      buf->nsize += MAX_LEN_PACKET;
-
-      /* added this check so we don't gobble up too much mem */
-      if (buf->nsize > MAX_LEN_BUFFER) {
-	if (delayed_disconnect > 0) {
-	  pc->delayed_disconnect = TRUE;
-	  return TRUE;
-	} else {
-	  if (close_callback) {
-	    (*close_callback)(pc);
-	  }
-	  return FALSE;
-	}
+    freelog(LOG_DEBUG, "add %d bytes to %d (space=%d)", len, buf->ndata,
+	    buf->nsize);
+    if (!buffer_ensure_free_extra_space(buf, len)) {
+      if (delayed_disconnect > 0) {
+	pc->delayed_disconnect = TRUE;
+	return TRUE;
       } else {
-	buf->data = (unsigned char *)fc_realloc(buf->data, buf->nsize);
+	if (close_callback) {
+	  (*close_callback) (pc);
+	}
+	return FALSE;
       }
     }
     memcpy(buf->data + buf->ndata, data, len);
@@ -314,6 +344,7 @@ void send_connection_data(struct connection *pc, const unsigned char *data,
 			  int len)
 {
   if (pc && pc->used) {
+    pc->statistics.bytes_send += len;
     if(pc->send_buffer->do_buffer_sends > 0) {
       flush_connection_send_buffer_packets(pc);
       if (!add_connection_data(pc, data, len)) {
@@ -353,7 +384,7 @@ void connection_do_unbuffer(struct connection *pc)
   if (pc && pc->used) {
     pc->send_buffer->do_buffer_sends--;
     if (pc->send_buffer->do_buffer_sends < 0) {
-      freelog(LOG_ERROR, "Too many calls to unbuffer %s!", pc->name);
+      freelog(LOG_ERROR, "Too many calls to unbuffer %s!", pc->username);
       pc->send_buffer->do_buffer_sends = 0;
     }
     if(pc->send_buffer->do_buffer_sends == 0)
@@ -378,38 +409,38 @@ void conn_list_do_unbuffer(struct conn_list *dest)
 }
 
 /***************************************************************
-  Find connection by exact name, from game.all_connections,
+  Find connection by exact user name, from game.all_connections,
   case-insensitve.  Returns NULL if not found.
 ***************************************************************/
-struct connection *find_conn_by_name(const char *name)
+struct connection *find_conn_by_user(const char *user_name)
 {
   conn_list_iterate(game.all_connections, pconn) {
-    if (mystrcasecmp(name, pconn->name)==0) {
+    if (mystrcasecmp(user_name, pconn->username)==0) {
       return pconn;
     }
-  }
-  conn_list_iterate_end;
+  } conn_list_iterate_end;
   return NULL;
 }
 
 /***************************************************************
-  Like find_conn_by_name(), but allow unambigous prefix
+  Like find_conn_by_username(), but allow unambigous prefix
   (ie abbreviation).
   Returns NULL if could not match, or if ambiguous or other
   problem, and fills *result with characterisation of
   match/non-match (see shared.[ch])
 ***************************************************************/
 static const char *connection_accessor(int i) {
-  return conn_list_get(&game.all_connections, i)->name;
+  return conn_list_get(&game.all_connections, i)->username;
 }
-struct connection *find_conn_by_name_prefix(const char *name,
-					    enum m_pre_result *result)
+
+struct connection *find_conn_by_user_prefix(const char *user_name,
+                                            enum m_pre_result *result)
 {
   int ind;
 
   *result = match_prefix(connection_accessor,
 			 conn_list_size(&game.all_connections),
-			 MAX_LEN_NAME-1, mystrncasecmp, name, &ind);
+			 MAX_LEN_NAME-1, mystrncasecmp, user_name, &ind);
   
   if (*result < M_PRE_AMBIGUOUS) {
     return conn_list_get(&game.all_connections, ind);
@@ -453,7 +484,7 @@ struct socket_packet_buffer *new_socket_packet_buffer(void)
 /**************************************************************************
   Free malloced struct
 **************************************************************************/
-void free_socket_packet_buffer(struct socket_packet_buffer *buf)
+static void free_socket_packet_buffer(struct socket_packet_buffer *buf)
 {
   if (buf) {
     if (buf->data) {
@@ -478,9 +509,9 @@ const char *conn_description(const struct connection *pconn)
 
   buffer[0] = '\0';
 
-  if (*pconn->name != '\0') {
+  if (*pconn->username != '\0') {
     my_snprintf(buffer, sizeof(buffer), _("%s from %s"),
-		pconn->name, pconn->addr); 
+		pconn->username, pconn->addr); 
   } else {
     sz_strlcpy(buffer, "server");
   }
@@ -515,3 +546,138 @@ int get_next_request_id(int old_request_id)
   assert(result);
   return result;
 }
+
+/**************************************************************************
+ ...
+**************************************************************************/
+void free_compression_queue(struct connection *pc)
+{
+#ifdef USE_COMPRESSION
+  byte_vector_free(&pc->compression.queue);
+#endif
+}
+
+/**************************************************************************
+ ...
+**************************************************************************/
+static void init_packet_hashs(struct connection *pc)
+{
+  enum packet_type i;
+
+  pc->phs.sent = fc_malloc(sizeof(*pc->phs.sent) * PACKET_LAST);
+  pc->phs.received = fc_malloc(sizeof(*pc->phs.received) * PACKET_LAST);
+  pc->phs.variant = fc_malloc(sizeof(*pc->phs.variant) * PACKET_LAST);
+
+  for (i = 0; i < PACKET_LAST; i++) {
+    pc->phs.sent[i] = NULL;
+    pc->phs.received[i] = NULL;
+    pc->phs.variant[i] = -1;
+  }
+}
+
+/**************************************************************************
+ ...
+**************************************************************************/
+static void free_packet_hashes(struct connection *pc)
+{
+  int i;
+
+  conn_clear_packet_cache(pc);
+
+  if (pc->phs.sent) {
+    for (i = 0; i < PACKET_LAST; i++) {
+      if (pc->phs.sent[i] != NULL) {
+	hash_free(pc->phs.sent[i]);
+	pc->phs.sent[i] = NULL;
+      }
+    }
+    free(pc->phs.sent);
+    pc->phs.sent = NULL;
+  }
+
+  if (pc->phs.received) {
+    for (i = 0; i < PACKET_LAST; i++) {
+      if (pc->phs.received[i] != NULL) {
+	hash_free(pc->phs.received[i]);
+	pc->phs.received[i] = NULL;
+      }
+    }
+    free(pc->phs.received);
+    pc->phs.received = NULL;
+  }
+
+  if (pc->phs.variant) {
+    free(pc->phs.variant);
+    pc->phs.variant = NULL;
+  }
+}
+
+/**************************************************************************
+ ...
+**************************************************************************/
+void connection_common_init(struct connection *pconn)
+{
+  pconn->established = FALSE;
+  pconn->used = TRUE;
+  pconn->last_write = 0;
+  pconn->buffer = new_socket_packet_buffer();
+  pconn->send_buffer = new_socket_packet_buffer();
+  pconn->statistics.bytes_send = 0;
+
+  init_packet_hashs(pconn);
+
+#ifdef USE_COMPRESSION
+  byte_vector_init(&pconn->compression.queue);
+#endif
+}
+
+/**************************************************************************
+ ...
+**************************************************************************/
+void connection_common_close(struct connection *pconn)
+{
+  if (!pconn->used) {
+    freelog(LOG_ERROR, "WARNING: Trying to close already closed connection");
+  } else {
+    my_closesocket(pconn->sock);
+    pconn->used = FALSE;
+    pconn->established = FALSE;
+
+    free_socket_packet_buffer(pconn->buffer);
+    pconn->buffer = NULL;
+
+    free_socket_packet_buffer(pconn->send_buffer);
+    pconn->send_buffer = NULL;
+
+    free_compression_queue(pconn);
+    free_packet_hashes(pconn);
+  }
+}
+
+/**************************************************************************
+ Remove all cached packets from the connection. This resets the
+ delta-state.
+**************************************************************************/
+void conn_clear_packet_cache(struct connection *pc)
+{
+  int i;
+
+  for (i = 0; i < PACKET_LAST; i++) {
+    if (pc->phs.sent != NULL && pc->phs.sent[i] != NULL) {
+      struct hash_table *hash = pc->phs.sent[i];
+      while (hash_num_entries(hash) > 0) {
+	const void *key = hash_key_by_number(hash, 0);
+	hash_delete_entry(hash, key);
+	free((void *) key);
+      }
+    }
+    if (pc->phs.received != NULL && pc->phs.received[i] != NULL) {
+      struct hash_table *hash = pc->phs.received[i];
+      while (hash_num_entries(hash) > 0) {
+	const void *key = hash_key_by_number(hash, 0);
+	hash_delete_entry(hash, key);
+	free((void *) key);
+      }
+    }
+  }
+}
