@@ -10,21 +10,23 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <assert.h>
+#include <stdlib.h>		/* exit */
+#include <string.h>
+#include <math.h>
 
 #include "fcintl.h"
 #include "game.h"
 #include "log.h"
+#include "mem.h"		/* free */
 #include "player.h"
+#include "shared.h"		/* ARRAY_SIZE */
 #include "support.h"
-#include "shared.h" /* ARRAY_SIZE */
 
 #include "tech.h"
 
@@ -33,10 +35,13 @@ struct advance advances[A_LAST];
    server/ruleset.c (for the server)
    client/packhand.c (for the client) */
 
+static int techcoststyle1[A_LAST];
+
 static const char *flag_names[] = {
   "Bonus_Tech", "Boat_Fast", "Bridge", "Railroad", "Fortress",
   "Watchtower", "Population_Pollution_Inc", "Trade_Revenue_Reduce",
-  "Airbase", "Farmland", "Reduce_Trireme_Loss1", "Reduce_Trireme_Loss2"
+  "Airbase", "Farmland", "Reduce_Trireme_Loss1", "Reduce_Trireme_Loss2", 
+  "Build_Airborne"
 };
 /* Note that these strings must correspond with the enums in tech_flag_id,
    in common/tech.h */
@@ -44,9 +49,10 @@ static const char *flag_names[] = {
 /**************************************************************************
 ...
 **************************************************************************/
-enum tech_state get_invention(struct player *pplayer, Tech_Type_id tech)
+enum tech_state get_invention(const struct player *pplayer,
+			      Tech_Type_id tech)
 {
-  assert(tech >= 0 || tech < game.num_tech_types);
+  assert(tech >= 0);
 
   return pplayer->research.inventions[tech].state;
 }
@@ -65,6 +71,7 @@ void set_invention(struct player *pplayer, Tech_Type_id tech,
 
   if (value == TECH_KNOWN) {
     game.global_advances[tech]++;
+    improvements_update_obsolete();
   }
 }
 
@@ -78,8 +85,7 @@ bool is_tech_a_req_for_goal(struct player *pplayer, Tech_Type_id tech,
   if (tech == goal) {
     return FALSE;
   } else {
-    return TEST_BIT(pplayer->research.inventions[goal].
-		    required_techs[tech / 8], tech % 8);
+    return BV_ISSET(pplayer->research.inventions[goal].required_techs, tech);
   }
 }
 
@@ -93,15 +99,23 @@ static void build_required_techs_helper(struct player *pplayer,
 {
   /* The is_tech_a_req_for_goal condition is true if the tech is
    * already marked */
-  if (tech == A_NONE || !tech_exists(tech)
+  if (!tech_is_available(pplayer, tech)
       || get_invention(pplayer, tech) == TECH_KNOWN
-      || is_tech_a_req_for_goal(pplayer, tech, goal)) {
+      || is_tech_a_req_for_goal(pplayer, tech, goal)
+      || tech == A_NONE) {
     return;
   }
 
   /* Mark the tech as required for the goal */
-  pplayer->research.inventions[goal].required_techs[tech / 8] |=
-      (1 << (tech % 8));
+  BV_SET(pplayer->research.inventions[goal].required_techs, tech);
+
+  if (advances[tech].req[0] == goal || advances[tech].req[1] == goal) {
+    /* TRANS: Obscure ruleset error */
+    freelog(LOG_FATAL, _("tech \"%s\": requires itself"),
+	    get_tech_name(pplayer, goal));
+    assert(0);
+    exit(EXIT_FAILURE);
+  }
 
   build_required_techs_helper(pplayer, advances[tech].req[0], goal);
   build_required_techs_helper(pplayer, advances[tech].req[1], goal);
@@ -113,11 +127,9 @@ static void build_required_techs_helper(struct player *pplayer,
 **************************************************************************/
 static void build_required_techs(struct player *pplayer, Tech_Type_id goal)
 {
-  Tech_Type_id i;
   int counter;
 
-  memset(pplayer->research.inventions[goal].required_techs, 0,
-	 sizeof(pplayer->research.inventions[goal].required_techs));
+  BV_CLR_ALL(pplayer->research.inventions[goal].required_techs);
 
   if (get_invention(pplayer, goal) == TECH_KNOWN) {
     pplayer->research.inventions[goal].num_required_techs = 0;
@@ -133,8 +145,8 @@ static void build_required_techs(struct player *pplayer, Tech_Type_id goal)
   pplayer->research.inventions[goal].num_required_techs = 1;
 
   counter = 0;
-  for (i = A_FIRST; i < game.num_tech_types; i++) {
-    if (!is_tech_a_req_for_goal(pplayer, i, goal)) {
+  tech_type_iterate(i) {
+    if (i == A_NONE || !is_tech_a_req_for_goal(pplayer, i, goal)) {
       continue;
     }
 
@@ -148,23 +160,51 @@ static void build_required_techs(struct player *pplayer, Tech_Type_id goal)
     pplayer->research.inventions[goal].num_required_techs++;
     pplayer->research.inventions[goal].bulbs_required +=
 	base_total_bulbs_required(pplayer, i);
-  }
+  } tech_type_iterate_end;
 
   /* Undo the changes made above */
   pplayer->research.techs_researched -= counter;
 }
 
 /**************************************************************************
-  Marks reachable techs. Calls build_required_techs to update the
-  cache of requirements.
+  Returns TRUE iff the given tech is ever reachable by the given player
+  by checking tech tree limitations.
+**************************************************************************/
+bool tech_is_available(struct player *pplayer, Tech_Type_id id)
+{
+  if (!tech_exists(id)) {
+    return FALSE;
+  }
+
+  if (advances[id].root_req != A_NONE
+      && get_invention(pplayer, advances[id].root_req) != TECH_KNOWN) {
+    /* This tech requires knowledge of another tech before being 
+     * available. Prevents sharing of untransferable techs. */
+    return FALSE;
+  }
+  return TRUE;
+}
+
+/**************************************************************************
+  Mark as TECH_REACHABLE each tech which is available, not known and
+  which has all requirements fullfiled.
+  If there is no such a tech mark A_FUTURE as researchable.
+  
+  Recalculate research->num_known_tech_with_flag
+  Should be called always after set_invention()
+
 **************************************************************************/
 void update_research(struct player *pplayer)
 {
-  Tech_Type_id i;
   enum tech_flag_id flag;
+  int researchable = 0;
 
-  for (i = 0; i < game.num_tech_types; i++) {
-    if (!tech_exists(i)) {
+  tech_type_iterate(i) {
+    if (i == A_NONE) {
+      /* This is set when the game starts, but not everybody finds out
+       * right away. */
+      set_invention(pplayer, i, TECH_KNOWN);
+    } else if (!tech_is_available(pplayer, i)) {
       set_invention(pplayer, i, TECH_UNKNOWN);
     } else {
       if (get_invention(pplayer, i) == TECH_REACHABLE) {
@@ -175,19 +215,26 @@ void update_research(struct player *pplayer)
 	  && get_invention(pplayer, advances[i].req[0]) == TECH_KNOWN
 	  && get_invention(pplayer, advances[i].req[1]) == TECH_KNOWN) {
 	set_invention(pplayer, i, TECH_REACHABLE);
+	researchable++;
       }
     }
     build_required_techs(pplayer, i);
+  } tech_type_iterate_end;
+
+  /* No techs we can research? Mark A_FUTURE as researchable */
+  if (researchable == 0) {
+    set_invention(pplayer, A_FUTURE, TECH_REACHABLE);
   }
 
+
   for (flag = 0; flag < TF_LAST; flag++) {
     pplayer->research.num_known_tech_with_flag[flag] = 0;
 
-    for (i = A_FIRST; i < game.num_tech_types; i++) {
+    tech_type_iterate(i) {
       if (get_invention(pplayer, i) == TECH_KNOWN && tech_flag(i, flag)) {
 	pplayer->research.num_known_tech_with_flag[flag]++;
       }
-    }
+    } tech_type_iterate_end;
   }
 }
 
@@ -199,14 +246,15 @@ static Tech_Type_id get_next_tech_rec(struct player *pplayer,
 {
   Tech_Type_id sub_goal;
 
-  if (!tech_exists(goal) || get_invention(pplayer, goal) == TECH_KNOWN) {
-    return A_NONE;
+  if (!tech_is_available(pplayer, goal)
+      || get_invention(pplayer, goal) == TECH_KNOWN) {
+    return A_UNSET;
   }
   if (get_invention(pplayer, goal) == TECH_REACHABLE) {
     return goal;
   }
   sub_goal = get_next_tech_rec(pplayer, advances[goal].req[0]);
-  if (sub_goal != A_NONE) {
+  if (sub_goal != A_UNSET) {
     return sub_goal;
   } else {
     return get_next_tech_rec(pplayer, advances[goal].req[1]);
@@ -221,9 +269,9 @@ static Tech_Type_id get_next_tech_rec(struct player *pplayer,
 **************************************************************************/
 Tech_Type_id get_next_tech(struct player *pplayer, Tech_Type_id goal)
 {
-  if (goal == A_NONE || !tech_exists(goal) ||
-      get_invention(pplayer, goal) == TECH_KNOWN) {
-    return A_NONE;
+  if (!tech_is_available(pplayer, goal)
+      || get_invention(pplayer, goal) == TECH_KNOWN) {
+    return A_UNSET;
   }
   return (get_next_tech_rec(pplayer, goal));
 }
@@ -238,10 +286,11 @@ A tech doesn't exist if one of:
 **************************************************************************/
 bool tech_exists(Tech_Type_id id)
 {
-  if (id<0 || id>=game.num_tech_types)
+  if (id < 0 || id >= game.num_tech_types) {
     return FALSE;
-  else 
-    return advances[id].req[0]!=A_LAST && advances[id].req[1]!=A_LAST;
+  } else {
+    return advances[id].req[0] != A_LAST && advances[id].req[1] != A_LAST;
+  }
 }
 
 /**************************************************************************
@@ -250,12 +299,25 @@ Returns A_LAST if none match.
 **************************************************************************/
 Tech_Type_id find_tech_by_name(const char *s)
 {
-  Tech_Type_id i;
-
-  for( i=0; i<game.num_tech_types; i++ ) {
-    if (strcmp(advances[i].name, s)==0)
+  tech_type_iterate(i) {
+    if (strcmp(advances[i].name, s) == 0) {
       return i;
-  }
+    }
+  } tech_type_iterate_end;
+  return A_LAST;
+}
+
+/**************************************************************************
+  Does a linear search of advances[].name_orig
+  Returns A_LAST if none match.
+**************************************************************************/
+Tech_Type_id find_tech_by_name_orig(const char *s)
+{
+  tech_type_iterate(i) {
+    if (mystrcasecmp(advances[i].name_orig, s) == 0) {
+      return i;
+    }
+  } tech_type_iterate_end;
   return A_LAST;
 }
 
@@ -264,7 +326,7 @@ Tech_Type_id find_tech_by_name(const char *s)
 **************************************************************************/
 bool tech_flag(Tech_Type_id tech, enum tech_flag_id flag)
 {
-  assert(flag>=0 && flag<TF_LAST);
+  assert(flag >= 0 && flag < TF_LAST);
   return TEST_BIT(advances[tech].flags, flag);
 }
 
@@ -300,27 +362,6 @@ Tech_Type_id find_tech_by_flag(int index, enum tech_flag_id flag)
   return A_LAST;
 }
 
-/**************************************************************************
- Returns number of turns to advance (assuming current state of
- civilization).
-**************************************************************************/
-int tech_turns_to_advance(struct player *pplayer)
-{
-  /* The number of bulbs the civilization produces every turn. */
-  int current_output = 0;
-
-  city_list_iterate(pplayer->cities, pcity) {
-    current_output += pcity->science_total;
-  } city_list_iterate_end;
-
-  if (current_output <= 0) {
-    return FC_INFINITY;
-  }
-
-  return ((total_bulbs_required(pplayer) + current_output - 1)
-	  / current_output);
-}
-
 /**************************************************************************
   Returns the number of bulbs which are required to finished the
   currently researched tech denoted by
@@ -341,8 +382,9 @@ int total_bulbs_required(struct player *pplayer)
  tech_cost_style:
  0 - Civ (I|II) style. Every new tech add game.researchcost to the
      cost of the next tech.
- 1 - Cost of technology is (1+num_parents)*researchcost, where
-     num_parents == number of requirement for tech, counted recursively.
+ 1 - Cost of technology is 
+       MAX((1 + parents) * (researchcost / 2) * sqrt(1 + parents), researchcost)
+     where num_parents == number of requirement for tech, counted recursively.
  2 - Cost are read from tech.ruleset. Missing costs are generated by
      style 1.
 
@@ -379,21 +421,21 @@ int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
     cost = pplayer->research.techs_researched * game.researchcost;
     break;
   case 1:
-    cost = advances[tech].num_reqs * game.researchcost;
+    cost = techcoststyle1[tech];
     break;
   case 2:
     cost = (advances[tech].preset_cost * game.researchcost) /
 	GAME_DEFAULT_RESEARCHCOST;
     break;
   default:
-    freelog(LOG_ERROR, "Invalid tech_cost_style %d %d",
-	    game.rgame.tech_cost_style, tech_cost_style);
-    assert(0);
-    exit(EXIT_FAILURE);
+    die("Invalid tech_cost_style %d %d", game.rgame.tech_cost_style,
+	tech_cost_style);
+    cost = 0;
   }
 
-  /* Research becomes more expensive. */
-  if (game.year > 0) {
+  /* Research becomes more expensive this year and after. */
+  if (game.rgame.tech_cost_double_year != 0
+      && game.year >= game.rgame.tech_cost_double_year) {
     cost *= 2;
   }
 
@@ -452,9 +494,18 @@ int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
     break;
 
   default:
-    freelog(LOG_ERROR, "Invalid tech_leakage %d", game.rgame.tech_leakage);
-    assert(0);
-    exit(EXIT_FAILURE);
+    die("Invalid tech_leakage %d", game.rgame.tech_leakage);
+  }
+
+  /* Assign a science penalty to the AI at easier skill levels.  This code
+   * can also be adpoted to create an extra-hard AI skill level where the AI
+   * gets science benefits */
+
+  /* FIXME: the is_server is a kludge to unbreak CVS. GB promises 
+   * to fix correctly a.s.a.p. 2003/5/18 -mck */
+  if (is_server && pplayer->ai.control) {
+    assert(pplayer->ai.science_cost > 0);
+    cost = (cost * pplayer->ai.science_cost) / 100;
   }
 
   /* If we have many players, tech cost may drop to 0.  */
@@ -508,13 +559,19 @@ static int precalc_tech_data_helper(Tech_Type_id tech, bool *counted)
 **************************************************************************/
 void precalc_tech_data()
 {
-  Tech_Type_id tech;
   bool counted[A_LAST];
 
-  for (tech = A_FIRST; tech < game.num_tech_types; tech++) {
+  tech_type_iterate(tech) {
     memset(counted, 0, sizeof(counted));
     advances[tech].num_reqs = precalc_tech_data_helper(tech, counted);
-  }
+  } tech_type_iterate_end;
+
+  tech_type_iterate(tech) {
+    techcoststyle1[tech] = MAX((advances[tech].num_reqs + 1)
+                               * sqrt(advances[tech].num_reqs + 1)
+                               * (game.researchcost / 2),
+                               game.researchcost);
+  } tech_type_iterate_end;
 }
 
 /**************************************************************************
@@ -522,29 +579,55 @@ void precalc_tech_data()
 **************************************************************************/
 bool is_future_tech(Tech_Type_id tech)
 {
-  /*
-   * Future techs can be identify in two ways: the "tech >=
-   * game.num_tech_types" condition and the "tech == A_NONE"
-   * condition. FIXME: clean this up.
-   */
-  return (tech >= game.num_tech_types) || (tech == A_NONE);
+  return tech == A_FUTURE;
 }
 
+#define SPECVEC_TAG string
+#define SPECVEC_TYPE char *
+#include "specvec.h"
+
 /**************************************************************************
  Return the name of the given tech. You don't have to free the return
  pointer.
 **************************************************************************/
 const char *get_tech_name(struct player *pplayer, Tech_Type_id tech)
 {
-  static char buffer[200];
+  static struct string_vector future;
+  int i;
 
-  if (!is_future_tech(tech)) {
-    my_snprintf(buffer, sizeof(buffer), "%s", advances[tech].name);
-  } else {
-    my_snprintf(buffer, sizeof(buffer), _("Future Tech. %d"),
-		pplayer->future_tech + 1);
+  /* We don't return a static buffer because that would break anything that
+   * needed to work with more than one name at a time. */
+  switch (tech) {
+  case A_NOINFO:
+    /* TRANS: "Unknown" tech */
+    return _("(Unknown)");
+  case A_UNSET:
+    /* TRANS: "None" tech */
+    return _("None");
+  case A_FUTURE:
+    /* pplayer->future_tech == 0 means "Future Tech. 1". */
+    for (i = future.size; i <= pplayer->future_tech; i++) {
+      char *ptr = NULL;
+
+      string_vector_append(&future, &ptr);
+    }
+    if (!future.p[pplayer->future_tech]) {
+      char buffer[1024];
+
+      my_snprintf(buffer, sizeof(buffer), _("Future Tech. %d"),
+		  pplayer->future_tech + 1);
+      future.p[pplayer->future_tech] = mystrdup(buffer);
+    }
+    return future.p[pplayer->future_tech];
+  default:
+    /* Includes A_NONE */
+    if (!tech_exists(tech)) {
+      /* TRANS: "Unknown" tech */
+      return _("(Unknown)");
+    } else {
+      return advances[tech].name;
+    }
   }
-  return buffer;
 }
 
 /**************************************************************************
@@ -561,7 +644,7 @@ bool techs_have_fixed_costs()
 /***************************************************************
  De-allocate resources associated with the given tech.
 ***************************************************************/
-void tech_free(Tech_Type_id tech)
+static void tech_free(Tech_Type_id tech)
 {
   struct advance *p = &advances[tech];
 
