@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -23,13 +24,12 @@
 #include "government.h"
 #include "mem.h"
 #include "player.h"
+#include "shared.h"
 #include "support.h"
 #include "tech.h"
-#include "shared.h" /* ARRAY_SIZE */
 
 #include "unittype.h"
 
-
 struct unit_type unit_types[U_LAST];
 /* the unit_types array is now setup in:
    server/ruleset.c (for the server)
@@ -45,14 +45,17 @@ static const char *flag_names[] = {
   "AEGIS", "Fighter", "Marines", "Partial_Invis", "Settlers", "Diplomat",
   "Trireme", "Nuclear", "Spy", "Transform", "Paratroopers",
   "Airbase", "Cities", "IgTired", "Missile_Carrier", "No_Land_Attack",
-  "AddToCity", "Fanatic"
+  "AddToCity", "Fanatic", "GameLoss", "Unique", "Unbribable", 
+  "Undisbandable", "SuperSpy", "NoHome", "NoVeteran", "Bombarder",
+  "CityBuster", "NoBuild"
 };
 static const char *role_names[] = {
   "FirstBuild", "Explorer", "Hut", "HutTech", "Partisan",
   "DefendOk", "DefendGood", "AttackFast", "AttackStrong",
   "Ferryboat", "Barbarian", "BarbarianTech", "BarbarianBoat",
   "BarbarianBuild", "BarbarianBuildTech", "BarbarianLeader",
-  "BarbarianSea", "BarbarianSeaTech"
+  "BarbarianSea", "BarbarianSeaTech", "Cities", "Settlers",
+  "GameLoss", "Diplomat"
 };
 static const char *unit_class_names[] = {
   "Air",
@@ -83,6 +86,7 @@ bool unit_type_exists(Unit_Type_id id)
 **************************************************************************/
 struct unit_type *get_unit_type(Unit_Type_id id)
 {
+  assert(id >= 0 && id < U_LAST && id < game.num_unit_types);
   return &unit_types[id];
 }
 
@@ -132,7 +136,9 @@ bool is_water_unit(Unit_Type_id id)
 int utype_shield_cost(struct unit_type *ut, struct government *g)
 {
   if (government_has_flag(g, G_FANATIC_TROOPS) &&
-      TEST_BIT(ut->flags, F_FANATIC)) return 0;
+      BV_ISSET(ut->flags, F_FANATIC)) {
+    return 0;
+  }
   return ut->shield_cost * g->unit_shield_cost_factor;
 }
 
@@ -166,7 +172,7 @@ int utype_gold_cost(struct unit_type *ut, struct government *g)
 bool unit_type_flag(Unit_Type_id id, int flag)
 {
   assert(flag>=0 && flag<F_LAST);
-  return TEST_BIT(unit_types[id].flags, flag);
+  return BV_ISSET(unit_types[id].flags, flag);
 }
 
 /**************************************************************************
@@ -183,15 +189,39 @@ bool unit_flag(struct unit *punit, enum unit_flag_id flag)
 bool unit_has_role(Unit_Type_id id, int role)
 {
   assert(role>=L_FIRST && role<L_LAST);
-  return TEST_BIT(unit_types[id].roles, (role - L_FIRST));
+  return BV_ISSET(unit_types[id].roles, role - L_FIRST);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-int unit_value(Unit_Type_id id)
+/****************************************************************************
+  Returns the number of shields it takes to build this unit.
+****************************************************************************/
+int unit_build_shield_cost(Unit_Type_id id)
 {
-  return (unit_types[id].build_cost);
+  return unit_types[id].build_cost;
+}
+
+/****************************************************************************
+  Returns the amount of gold it takes to rush this unit.
+****************************************************************************/
+int unit_buy_gold_cost(Unit_Type_id id, int shields_in_stock)
+{
+  int cost = 0, missing = unit_types[id].build_cost - shields_in_stock;
+
+  if (missing > 0) {
+    cost = 2 * missing + (missing * missing) / 20;
+  }
+  if (shields_in_stock == 0) {
+    cost *= 2;
+  }
+  return cost;
+}
+
+/****************************************************************************
+  Returns the number of shields received when this unit is disbanded.
+****************************************************************************/
+int unit_disband_shields(Unit_Type_id id)
+{
+  return unit_types[id].build_cost / 2;
 }
 
 /**************************************************************************
@@ -203,18 +233,24 @@ int unit_pop_value(Unit_Type_id id)
 }
 
 /**************************************************************************
-...
+  Return the (translated) name of the unit type.
 **************************************************************************/
-
-char *unit_name(Unit_Type_id id)
+const char *unit_name(Unit_Type_id id)
 {
   return (unit_types[id].name);
 }
 
+/**************************************************************************
+  Return the original (untranslated) name of the unit type.
+**************************************************************************/
+const char *unit_name_orig(Unit_Type_id id)
+{
+  return unit_types[id].name_orig;
+}
 /**************************************************************************
 ...
 **************************************************************************/
-char *get_unit_name(Unit_Type_id id)
+const char *get_unit_name(Unit_Type_id id)
 {
   struct unit_type *ptype;
   static char buffer[256];
@@ -252,39 +288,43 @@ const char *unit_class_name(Unit_Class_id id)
  TODO: if there are more than 4 units with this flag return
        a fallback string (e.g. first unit name + "and similar units"
 **************************************************************************/
-char *get_units_with_flag_string(int flag)
+const char *get_units_with_flag_string(int flag)
 {
-  int count=num_role_units(flag);
+  int count = num_role_units(flag);
 
-  if(count==1)
-    return mystrdup(unit_name(get_role_unit(flag,0)));
+  if (count == 1) {
+    return mystrdup(unit_name(get_role_unit(flag, 0)));
+  }
 
-  if(count > 0) {
+  if (count > 0) {
     struct astring astr;
 
     astr_init(&astr);
-    astr_minsize(&astr,1);
+    astr_minsize(&astr, 1);
     astr.str[0] = 0;
 
-    while((count--) > 0) {
-      int u = get_role_unit(flag,count);
-      char *unitname = unit_name(u);
+    while ((count--) > 0) {
+      int u = get_role_unit(flag, count);
+      const char *unitname = unit_name(u);
 
       /* there should be something like astr_append() */
-      astr_minsize(&astr,astr.n+strlen(unitname));
-      strcat(astr.str,unitname);
+      astr_minsize(&astr, astr.n + strlen(unitname));
+      strcat(astr.str, unitname);
 
-      if(count==1) {
-	char *and_str = _(" and ");
-        astr_minsize(&astr,astr.n+strlen(and_str));
+      if (count == 1) {
+	const char *and_str = _(" and ");
+
+        astr_minsize(&astr, astr.n + strlen(and_str));
         strcat(astr.str, and_str);
-      }
-      else {
-        if(count != 0) {
-          astr_minsize(&astr,astr.n+strlen(", "));
-          strcat(astr.str,", ");
-        }
-        else return astr.str;
+      } else {
+        if (count != 0) {
+	  const char *and_comma = Q_("?and:, ");
+
+	  astr_minsize(&astr, astr.n + strlen(and_comma));
+	  strcat(astr.str, and_comma);
+        } else {
+	  return astr.str;
+	}
       }
     }
   }
@@ -308,28 +348,44 @@ int can_upgrade_unittype(struct player *pplayer, Unit_Type_id id)
 }
 
 /**************************************************************************
-...
+  Return the cost (gold) of upgrading a single unit of the specified type
+  to the new type.  This price could (but currently does not) depend on
+  other attributes (like nation or government type) of the player the unit
+  belongs to.
 **************************************************************************/
-int unit_upgrade_price(struct player *pplayer, Unit_Type_id from,
-		       Unit_Type_id to)
+int unit_upgrade_price(const struct player *const pplayer,
+		       const Unit_Type_id from, const Unit_Type_id to)
 {
-  int total, build;
-  build = unit_value(from)/2;
-  total = unit_value(to);
-  if (build>=total)
-    return 0;
-  return (total-build)*2+(total-build)*(total-build)/20; 
+  return unit_buy_gold_cost(to, unit_disband_shields(from));
 }
 
 /**************************************************************************
-Does a linear search of unit_types[].name
-Returns U_LAST if none match.
+  Returns the unit type that has the given (translated) name.
+  Returns U_LAST if none match.
 **************************************************************************/
-Unit_Type_id find_unit_type_by_name(char *s)
+Unit_Type_id find_unit_type_by_name(const char *name)
 {
+  /* Does a linear search of unit_types[].name */
   unit_type_iterate(i) {
-    if (strcmp(unit_types[i].name, s)==0)
+    if (strcmp(unit_types[i].name, name) == 0) {
       return i;
+    }
+  } unit_type_iterate_end;
+
+  return U_LAST;
+}
+
+/**************************************************************************
+  Returns the unit type that has the given original (untranslated) name.
+  Returns U_LAST if none match.
+**************************************************************************/
+Unit_Type_id find_unit_type_by_name_orig(const char *name_orig)
+{
+  /* Does a linear search of unit_types[].name_orig. */
+  unit_type_iterate(i) {
+    if (mystrcasecmp(unit_types[i].name_orig, name_orig) == 0) {
+      return i;
+    }
   } unit_type_iterate_end;
 
   return U_LAST;
@@ -339,7 +395,7 @@ Unit_Type_id find_unit_type_by_name(char *s)
   Convert unit_move_type names to enum; case insensitive;
   returns 0 if can't match.
 **************************************************************************/
-enum unit_move_type unit_move_type_from_str(char *s)
+enum unit_move_type unit_move_type_from_str(const char *s)
 {
   enum unit_move_type i;
 
@@ -358,7 +414,7 @@ enum unit_move_type unit_move_type_from_str(char *s)
   Convert Unit_Class_id names to enum; case insensitive;
   returns UCL_LAST if can't match.
 **************************************************************************/
-Unit_Class_id unit_class_from_str(char *s)
+Unit_Class_id unit_class_from_str(const char *s)
 {
   Unit_Class_id i;
 
@@ -376,7 +432,7 @@ Unit_Class_id unit_class_from_str(char *s)
   Convert flag names to enum; case insensitive;
   returns F_LAST if can't match.
 **************************************************************************/
-enum unit_flag_id unit_flag_from_str(char *s)
+enum unit_flag_id unit_flag_from_str(const char *s)
 {
   enum unit_flag_id i;
 
@@ -394,7 +450,7 @@ enum unit_flag_id unit_flag_from_str(char *s)
   Convert role names to enum; case insensitive;
   returns L_LAST if can't match.
 **************************************************************************/
-enum unit_role_id unit_role_from_str(char *s)
+enum unit_role_id unit_role_from_str(const char *s)
 {
   enum unit_role_id i;
 
@@ -415,15 +471,42 @@ player has a coastal city.
 **************************************************************************/
 bool can_player_build_unit_direct(struct player *p, Unit_Type_id id)
 {
+  Impr_Type_id impr_req;
+  Tech_Type_id tech_req;
+
   if (!unit_type_exists(id))
     return FALSE;
-  if (unit_type_flag(id, F_NUCLEAR) && game.global_wonders[B_MANHATTEN] == 0)
+  if (unit_type_flag(id, F_NUCLEAR)
+      && !get_player_bonus(p, EFT_ENABLE_NUKE) > 0)
     return FALSE;
+  if (unit_type_flag(id, F_NOBUILD)) {
+    return FALSE;
+  }
   if (unit_type_flag(id, F_FANATIC)
       && !government_has_flag(get_gov_pplayer(p), G_FANATIC_TROOPS))
     return FALSE;
   if (get_invention(p,unit_types[id].tech_requirement)!=TECH_KNOWN)
     return FALSE;
+  if (unit_type_flag(id, F_UNIQUE)) {
+    /* FIXME: This could be slow if we have lots of units. We could
+     * consider keeping an array of unittypes updated with this info 
+     * instead. */
+    unit_list_iterate(p->units, punit) {
+      if (punit->type == id) { 
+        return FALSE;
+      }
+    } unit_list_iterate_end;
+  }
+
+  /* If the unit has a building requirement, we check to see if the player
+   * can build that building.  Note that individual cities may not have
+   * that building, so they still may not be able to build the unit. */
+  impr_req = unit_types[id].impr_requirement;
+  tech_req = get_improvement_type(impr_req)->tech_req;
+  if (impr_req != B_LAST && get_invention(p, tech_req) != TECH_KNOWN) {
+    return FALSE;
+  }
+
   return TRUE;
 }
 
@@ -448,11 +531,17 @@ with future tech.  returns 0 if unit is obsolete.
 **************************************************************************/
 bool can_player_eventually_build_unit(struct player *p, Unit_Type_id id)
 {
-  if (!unit_type_exists(id))
+  if (!unit_type_exists(id)) {
     return FALSE;
-  while(unit_type_exists((id = unit_types[id].obsoleted_by)))
-    if (can_player_build_unit_direct(p, id))
+  }
+  if (unit_type_flag(id, F_NOBUILD)) {
+    return FALSE;
+  }
+  while (unit_type_exists((id = unit_types[id].obsoleted_by))) {
+    if (can_player_build_unit_direct(p, id)) {
 	return FALSE;
+    }
+  }
   return TRUE;
 }
 
@@ -544,7 +633,7 @@ Unit_Type_id get_role_unit(int role, int index)
 
 /**************************************************************************
 Return "best" unit this city can build, with given role/flag.
-Returns U_LAST if none match.
+Returns U_LAST if none match. "Best" means highest unit type id.
 **************************************************************************/
 Unit_Type_id best_role_unit(struct city *pcity, int role)
 {
@@ -562,10 +651,54 @@ Unit_Type_id best_role_unit(struct city *pcity, int role)
   return U_LAST;
 }
 
+/**************************************************************************
+Return "best" unit the player can build, with given role/flag.
+Returns U_LAST if none match. "Best" means highest unit type id.
+
+TODO: Cache the result per player?
+**************************************************************************/
+Unit_Type_id best_role_unit_for_player(struct player *pplayer, int role)
+{
+  int j;
+
+  assert((role >= 0 && role < F_LAST) || (role >= L_FIRST && role < L_LAST));
+
+  for(j = n_with_role[role]-1; j >= 0; j--) {
+    Unit_Type_id utype = with_role[role][j];
+
+    if (can_player_build_unit(pplayer, utype)) {
+      return utype;
+    }
+  }
+
+  return U_LAST;
+}
+
+/**************************************************************************
+  Return first unit the player can build, with given role/flag.
+  Returns U_LAST if none match.  Used eg when placing starting units.
+**************************************************************************/
+Unit_Type_id first_role_unit_for_player(struct player *pplayer, int role)
+{
+  int j;
+
+  assert((role >= 0 && role < F_LAST) || (role >= L_FIRST && role < L_LAST));
+
+  for(j = 0; j < n_with_role[role]; j++) {
+    Unit_Type_id utype = with_role[role][j];
+
+    if (can_player_build_unit(pplayer, utype)) {
+      return utype;
+    }
+  }
+
+  return U_LAST;
+}
+
 /**************************************************************************
   Frees the memory associated with this unit type.
 **************************************************************************/
-void unit_type_free(Unit_Type_id id)
+static void unit_type_free(Unit_Type_id id)
 {
   struct unit_type *p = get_unit_type(id);
 
